#include <avr/io.h>
#include <avr/pgmspace.h>

#include "mydefs.h"
#include "spi.h"
#include "lcd6100.h"

//------------------------------------------------------------------------------
void lcdInit(void)
{
  unsigned char *p = (unsigned char *)0xD000;

  unsigned int i;
  unsigned int j;
  
  spiInit();

  SPCR &= ~_BV(SPE);
  
  PORTB &= 0xF9;
  PORTB |= _BV(PB7);
  
  *p = 0;
  for(i=0;i<100;i++)
  {
    _delay_ms(5);
  }
  p = (unsigned char *)0x8000;
  *p = 0;
  for(i=0;i<100;i++)
  {
    _delay_ms(5);
  }

  SPCR |= _BV(SPE);

  lcdWriteByte(DISCTL,0);  // display control
  lcdWriteByte(0x0C,1);
  lcdWriteByte(0x20,1);
  lcdWriteByte(0x0C,1);
  lcdWriteByte(0x00,1);

  lcdWriteByte(COMSCN,0);
  lcdWriteByte(0x01,1);
  
  lcdWriteByte(OSCON,0);
  
  lcdWriteByte(SLPOUT,0);

  lcdWriteByte(VOLCTR,0);   // electronic volume, this is kinda contrast/brightness
  lcdWriteByte(0x08,1);     // this might be different for individual LCDs
  lcdWriteByte(0x03,1);
    
  lcdWriteByte(PWRCTR,0);
  lcdWriteByte(0x0F,1);
  
  for(i=0;i<200;i++)
  {
    _delay_ms(5);
  }

  lcdWriteByte(DISINV,0);
  
  lcdWriteByte(DATCTL,0);
  lcdWriteByte(0x00,1);
  lcdWriteByte(0x02,1);
  lcdWriteByte(0x01,1);

  lcdWriteByte(RGBSET8,0);   // setup color lookup table
    // color table
    //RED
  lcdWriteByte(0x00,1);
  lcdWriteByte(0x02,1);
  lcdWriteByte(0x04,1);
  lcdWriteByte(0x06,1);
  lcdWriteByte(0x09,1);
  lcdWriteByte(0x0B,1);
  lcdWriteByte(0x0D,1);
  lcdWriteByte(0x0F,1);
    // GREEN
  lcdWriteByte(0x00,1);
  lcdWriteByte(0x02,1);
  lcdWriteByte(0x04,1);
  lcdWriteByte(0x06,1);
  lcdWriteByte(0x08,1);
  lcdWriteByte(0x0B,1);
  lcdWriteByte(0x0D,1);
  lcdWriteByte(0x0F,1);
    //BLUE
  lcdWriteByte(0x00,1);
  lcdWriteByte(0x04,1);
  lcdWriteByte(0x09,1);
  lcdWriteByte(0x0F,1);

  lcdWriteByte(NOP,0);        // NOP

  lcdWriteByte(DISON,0);

//  lcdWriteByte(PASET,0);      // page start/end ram
//  lcdWriteByte(0x02,1);       // for some reason starts at 2
//  lcdWriteByte(0x84,1);          
//  
//  lcdWriteByte(CASET,0);      // column start/end ram
//  lcdWriteByte(0x02,1);          
//  lcdWriteByte(0x84,1);
//  
//  lcdWriteByte(RAMWR,0);      // write some stuff (background)
//  for (i = 0; i < 5461; i++)
//  {
//    lcdWriteByte(0x02,1);     // 28 is green
//  }
//  for (i = 0; i < 5462; i++)
//  {
//    lcdWriteByte(0x1C,1);     // 28 is green
//  }
//  for (i = 0; i < 5461; i++)
//  {
//    lcdWriteByte(0xE0,1);     // 28 is green
//  }
//
//  for(i=0;i<200;i++)
//  {
//    _delay_ms(5);
//  }
  
//  for (i = 0; i < 160; i++)   // this loop adjusts the contrast, change the number of iterations to get
//  {   
//    lcdWriteByte(VOLUP,0);    // desired contrast.  This might be different for individual LCDs
//
//    for(j=0;j<20;j++)
//    {
//      _delay_ms(5);
//    }
//  }  

//  lcdWriteByte(DISOFF,0);
}

//------------------------------------------------------------------------------
void lcdWriteByte(unsigned char byte, unsigned char data)
{
  SPCR &= ~_BV(SPE);
  
  PORTB &= 0xF9;
  if(data)
  {
    PORTB |= _BV(PB2);
  }
  else
  {
    PORTB &= ~_BV(PB2);
  }
  lcdClrCS();  
  
  _delay_us(1);
  PORTB |= _BV(PB1);
  _delay_us(1);
  PORTB &= ~_BV(PB1);
  _delay_us(1);
  PORTB &= ~_BV(PB2);
  
  SPCR |= _BV(SPE);
  
  spiWriteByte(byte);    

  lcdSetCS();
}

//------------------------------------------------------------------------------
void lcdSetAddress(unsigned char xs, 
                   unsigned char ys, 
                   unsigned char xe, 
                   unsigned char ye)
{
  lcdWriteByte(CASET,0);
  lcdWriteByte(xs,1);
  lcdWriteByte(xe,1);

  lcdWriteByte(PASET,0);
  lcdWriteByte(ys,1);
  lcdWriteByte(ye,1);
}

//------------------------------------------------------------------------------
void lcdPutPixel(unsigned char color, 
                 unsigned char x,
                 unsigned char y)
{
  lcdSetAddress(x,y,X_END,Y_END);
  lcdWriteByte(RAMWR,0);
  lcdWriteByte(color,1);
}

//------------------------------------------------------------------------------
void lcdPutArrayFlash(unsigned char *data, 
                      unsigned char x, 
                      unsigned char y,
                      unsigned char width,
                      unsigned char height)
{
  unsigned int i;
  unsigned int pixel_count = width * height;
  
  lcdSetAddress(x,y,x+width-1,y+height);
  lcdWriteByte(RAMWR,0);
  for(i=0;i<pixel_count;i++)
  {
    lcdWriteByte(pgm_read_byte_near(data+ i),1);
  }
}

//------------------------------------------------------------------------------
void lcdClearScreen(unsigned char color)
{
  unsigned int i;
  
  lcdSetAddress(0,0,X_END-1,Y_END-1);
  lcdWriteByte(RAMWR,0);
  for(i=0;i<MAX_PIXEL;i++)
  {
    lcdWriteByte(color,1);
  }
}

//------------------------------------------------------------------------------
void lcdSetContrast(unsigned char data)
{
  lcdWriteByte(0x5C,0);
  lcdWriteByte(data,1);
}
